==> better-with-aether-0.9.0%2B7.3_04-sources.jar: teamport/aether/entity/monster/mimic/MobMimic.java <==
package teamport.aether.entity.monster.mimic;

import com.mojang.nbt.tags.CompoundTag;
import net.minecraft.core.Global;
import net.minecraft.core.WeightedRandomLootObject;
import net.minecraft.core.block.Block;
import net.minecraft.core.block.BlockLogic;
import net.minecraft.core.block.BlockLogicChest;
import net.minecraft.core.block.Blocks;
import net.minecraft.core.block.entity.TileEntity;
import net.minecraft.core.block.material.Material;
import net.minecraft.core.entity.Entity;
import net.minecraft.core.entity.EntityDispatcher;
import net.minecraft.core.entity.monster.Enemy;
import net.minecraft.core.entity.player.Player;
import net.minecraft.core.item.ItemStack;
import net.minecraft.core.item.tool.ItemToolAxe;
import net.minecraft.core.item.tool.ItemToolPickaxe;
import net.minecraft.core.player.inventory.container.Container;
import net.minecraft.core.util.collection.NamespaceID;
import net.minecraft.core.util.helper.DamageType;
import net.minecraft.core.util.helper.Direction;
import net.minecraft.core.util.helper.DyeColor;
import net.minecraft.core.world.World;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import teamport.aether.blocks.AetherBlockTags;
import teamport.aether.blocks.AetherBlocks;
import teamport.aether.blocks.dungeon.BlockLogicChestMimic;
import teamport.aether.entity.AetherDeathMessage;
import teamport.aether.entity.monster.MobMonsterAether;
import teamport.aether.entity.tile.TileEntityMimic;
import teamport.aether.helper.unboxed.IntPair;
import teamport.aether.items.accessory.AetherInvisibility;
import teamport.aether.items.itemtool.ItemToolAxeAether;
import teamport.aether.items.itemtool.ItemToolPickaxeAether;
import teamport.aether.world.generate.feature.components.WorldFeatureComponent;
import teamport.aether.world.generate.feature.components.WorldFeaturePoint;

import java.util.*;

import static net.minecraft.core.net.command.TextFormatting.RED;
import static net.minecraft.core.net.command.TextFormatting.RESET;
import static net.minecraft.core.util.helper.Direction.*;
import static teamport.aether.AetherMod.TRANSLATOR;
import static teamport.aether.helper.unboxed.IntPair.ipair;
import static teamport.aether.world.generate.feature.components.WorldFeaturePoint.wfp;

public class MobMimic extends MobMonsterAether implements Enemy, AetherDeathMessage {
    public int mimicTime;
    int mimicChestID = AetherBlocks.CHEST_MIMIC_SKYROOT.id();
    int mimicChestMetadata = 0;

    public MobMimic(World world) {
        super(world);
        this.setSize(1.0F, 2.0F);
        this.attackStrength = 5;
        this.scoreValue = 2000;
        this.mimicTime = 60 * Global.TICKS_PER_SECOND; //temp set to 2, was 120
        this.textureIdentifier = NamespaceID.getPermanent("aether", "mimic");
        this.setSkinVariant(this.getSkinVariant());
    }

    public void setBlockData(int mimicChestID, int mimicChestMetadata) {
        this.mimicChestID = mimicChestID;
        this.mimicChestMetadata = mimicChestMetadata;
    }

    public void setChatColor(byte chatColor) {
        this.chatColor = chatColor;
    }

    @Override
    public void addAdditionalSaveData(@NotNull CompoundTag tag) {
        super.addAdditionalSaveData(tag);
        tag.putInt("MimicChestID", mimicChestID);
        tag.putInt("MimicChestMetadata", mimicChestMetadata);
    }

    @Override
    public void readAdditionalSaveData(@NotNull CompoundTag tag) {
        super.readAdditionalSaveData(tag);
        this.mimicChestID = tag.getShort("MimicChestID");
        this.mimicChestMetadata = tag.getShort("MimicChestMetadata");
    }

    @Override
    public void dropDeathItems() {
        MimicEntry variant = MimicRegistry.getMimicVariantByID(this.getSkinVariant());
        this.dropItem(new ItemStack(variant.getChestID(), 1, variant.getChestMetadata()), 0);
        for (WeightedRandomLootObject lootObject : mobDrops) {
            ItemStack stack = lootObject.getDefinedItemStack();
            if (stack != null) {
                this.dropItem(stack, 0);
            }
        }
    }

    @Override
    public void updateAI() {
        super.updateAI();
        if (target == null) {
            if (mimicTime-- == 0) {
                this.remove();
            }
        }
    }

    @Override
    public Entity findPlayerToAttack() {
        assert this.world != null;
        Player player = this.world.getClosestPlayerToEntity(this, 64.0);
        if (player == null || !this.canEntityBeSeen(player) || !player.getGamemode().areMobsHostile()) {
            return null;
        }
        if (!(player instanceof AetherInvisibility)) {
            return player;
        }
        AetherInvisibility invPlayer = (AetherInvisibility) player;
        if (invPlayer.aether$isInvisible()) {
            Player newPlayer = this.world.getClosestPlayerToEntity(this, 2.0);
            if (newPlayer == null || !this.canEntityBeSeen(newPlayer) || !newPlayer.getGamemode().areMobsHostile()) {
                return null;
            }
            return newPlayer;
        }
        return player;
    }

    public void attackEntity(@NotNull Entity entity, float distance) {
        if (this.attackTime <= 0 && distance < 2.0F && entity.bb.maxY > this.bb.minY && entity.bb.minY < this.bb.maxY) {
            this.attackTime = 20;

            int attack = this.attackStrength;
            if (isWallace()) attack = (int) (attack * 1.5);

            entity.hurt(this, attack, DamageType.COMBAT);
        }

    }

    @Override
    public boolean hurt(Entity attacker, int damage, DamageType type) {
        if (damage > 0 && this.isWallace()) damage = Math.max(1, damage / 3);
        return super.hurt(attacker, this.extraDamage(attacker, damage), type);
    }

    public int extraDamage(Entity attacker, int damage) {
        if (!(attacker instanceof Player)) {
            return damage;
        }
        ItemStack item = ((Player) attacker).inventory.getCurrentItem();
        if (item == null) {
            return damage;
        }
        Block<?> block = Blocks.getBlock(this.mimicChestID);
        if (block == null) {
            return damage;
        }
        if (block.hasTag(AetherBlockTags.MINEABLE_BY_AETHER_AXE)
                && (item.getItem() instanceof ItemToolAxe || item.getItem() instanceof ItemToolAxeAether)
        ) {
            return damage << 1;
        }
        if (block.hasTag(AetherBlockTags.MINEABLE_BY_AETHER_PICKAXE)
                && (item.getItem() instanceof ItemToolPickaxe || item.getItem() instanceof ItemToolPickaxeAether)) {
            return damage << 1;
        }
        return damage;
    }

    @Override
    public String getHurtSound() {
        Block<?> block = Blocks.getBlock(mimicChestID);
        if (block == null) {
            return "step.wood";
        }
        return block.getSound().getStepSoundName();
    }

    @Override
    public String getDeathSound() {
        Block<?> block = Blocks.getBlock(mimicChestID);
        Material material = block == null ? Material.wood : block.getMaterial();
        if (material == Material.stone) {
            return "step.stone";
        }
        return "random.door_open";
    }

    @Override
    public float getSoundVolume() {
        return 0.6F;
    }

    @Override
    public int getMaxHealth() {
        return 80;
    }

    public void setLoot(List<ItemStack> loot) {
        if (loot == null || loot.isEmpty()) return;
        for (ItemStack itemStack : loot) {
            mobDrops.add(new WeightedRandomLootObject(itemStack));
        }
    }

    @Override
    public void remove() {
        if (this.isAlive()) {
            place();
        }
        super.remove();
    }

    @Override
    public String deathMessage(Player player) {
        String key = EntityDispatcher.nameKeyForClass(((Entity) this).getClass()) + ".death_message";
        String deathMessage = TRANSLATOR
                .translateKey(key)
                .replace("[PLAYER]", RESET + String.format("<%s>", player.getDisplayName()) + RESET + RED);
        return RED + deathMessage;
    }

    @Override
    public float getHeadHeight() {
        return this.bbHeight;
    }

    private void place() {
        WorldFeaturePoint point = wfp((int) Math.round(this.x), (int) Math.round(this.y), (int) Math.round(this.z));
        Direction[] check = new Direction[]{NONE, NORTH, EAST, SOUTH, WEST, UP, DOWN};
        for (Direction dir : check) {
            WorldFeaturePoint dPoint = point.copy().moveInDirection(dir);
            if (this.isSafe(world, dPoint)) {
                this.placeChest(dPoint);
                this.populateChest(dPoint);
                return;
            }
        }
        this.placeChest(point);
        this.populateChest(point);
    }

    private boolean isSafe(@Nullable World world, WorldFeaturePoint point) {
        Block<?> block = world.getBlock(point.x, point.y, point.z);
        int blockID = block == null ? 0 : block.id();
        Material blockMaterial = blockID == 0 ? Material.air : block.getMaterial();
        return blockID == 0 || blockMaterial.isLiquid();
    }

    private void placeChest(WorldFeaturePoint point) {
        IntPair blockAndMeta = getTarget(world, point);
        world.setBlockAndMetadataWithNotify(point.x, point.y, point.z, blockAndMeta.first, blockAndMeta.second);
        BlockLogicChestMimic.setDefaultDirection(world, point.x, point.y, point.z);
        TileEntity tileEntity = world.getTileEntity(point.x, point.y, point.z);
        if (tileEntity instanceof TileEntityMimic)
            ((TileEntityMimic) tileEntity).setCustomName(this.nickname, this.chatColor);
    }

    private IntPair getTarget(World world, WorldFeaturePoint point) {
        Map<WorldFeaturePoint, Integer> distance = new HashMap<>();
        Queue<WorldFeaturePoint> queue = new ArrayDeque<>();
        Direction[] check = new Direction[]{NORTH, EAST, SOUTH, WEST, UP, DOWN};
        queue.add(point);
        distance.put(point, 0);
        while (!queue.isEmpty()) {
            WorldFeaturePoint next = queue.poll();
            int cdist = distance.get(next);
            if (cdist >= 5) break;
            for (Direction direction : check) {
                WorldFeaturePoint to = new WorldFeaturePoint(next.x + direction.getOffsetX(), next.y + direction.getOffsetY(), next.z + direction.getOffsetZ());
                if (distance.getOrDefault(to, -1) != -1) continue;
                distance.put(to, cdist + 1);
                Block<?> block = world.getBlock(to.x, to.y, to.z);
                int metadata = world.getBlockMetadata(to.x, to.y, to.z);
                BlockLogic blockLogic = block == null ? null : block.getLogic();
                if (blockLogic instanceof BlockLogicChestMimic) {
                    return ipair(block.id(), metadata);
                }
                if (blockLogic instanceof BlockLogicChest) {
                    MimicEntry variant = MimicRegistry.getMimicVariantByChest(block.id(), metadata &= 240);
                    return ipair(variant.getMimicChestId(), variant.getMimicChestMetadata());
                }
                queue.add(to);
            }
        }
        MimicEntry variant = MimicRegistry.getMimicVariantByID(this.getSkinVariant());
        return ipair(variant.getMimicChestId(), variant.getMimicChestMetadata());
    }

    private void populateChest(WorldFeaturePoint point) {
        Container inventory = BlockLogicChest.getInventory(world, point.x, point.y, point.z);
        if (inventory == null) {
            return;
        }
        List<WeightedRandomLootObject> listLootObj = this.getMobDrops();
        for (WeightedRandomLootObject lootObj : listLootObj) {
            WorldFeatureComponent.placeItemInChest(random, lootObj.getDefinedItemStack(), inventory);
        }
    }

    public boolean isWallace() {
        MimicEntry variantWallace = MimicRegistry.getMimicVariantByChest(
                AetherBlocks.CHEST_PLANKS_SKYROOT_PAINTED.id(),
                AetherBlocks.CHEST_PLANKS_SKYROOT_PAINTED.getLogic().toMetadata(DyeColor.PURPLE)
        );

        return "Wallace".equals(nickname)
                && variantWallace.getMimicVariant() == this.getSkinVariant();
    }
}

==> better-with-aether-0.10.0%2B7.3_04-sources.jar: teamport/aether/entity/monster/mimic/MobMimic.java <==
package teamport.aether.entity.monster.mimic;

import com.mojang.nbt.tags.CompoundTag;
import net.minecraft.core.Global;
import net.minecraft.core.WeightedRandomLootObject;
import net.minecraft.core.block.Block;
import net.minecraft.core.block.BlockLogic;
import net.minecraft.core.block.BlockLogicChest;
import net.minecraft.core.block.Blocks;
import net.minecraft.core.block.entity.TileEntity;
import net.minecraft.core.block.material.Material;
import net.minecraft.core.entity.Entity;
import net.minecraft.core.entity.EntityDispatcher;
import net.minecraft.core.entity.monster.Enemy;
import net.minecraft.core.entity.player.Player;
import net.minecraft.core.item.ItemStack;
import net.minecraft.core.item.tool.ItemToolAxe;
import net.minecraft.core.item.tool.ItemToolPickaxe;
import net.minecraft.core.player.inventory.container.Container;
import net.minecraft.core.util.collection.NamespaceID;
import net.minecraft.core.util.helper.DamageType;
import net.minecraft.core.util.helper.Direction;
import net.minecraft.core.util.helper.DyeColor;
import net.minecraft.core.world.World;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import teamport.aether.blocks.AetherBlockTags;
import teamport.aether.blocks.AetherBlocks;
import teamport.aether.blocks.dungeon.BlockLogicChestMimic;
import teamport.aether.entity.AetherDeathMessage;
import teamport.aether.entity.monster.MobMonsterAether;
import teamport.aether.entity.tile.TileEntityMimic;
import teamport.aether.helper.unboxed.IntPair;
import teamport.aether.items.accessory.AetherInvisibility;
import teamport.aether.items.itemtool.ItemToolAxeAether;
import teamport.aether.items.itemtool.ItemToolPickaxeAether;
import teamport.aether.world.generate.feature.components.WorldFeatureComponent;
import teamport.aether.world.generate.feature.components.WorldFeaturePoint;

import java.util.*;

import static net.minecraft.core.net.command.TextFormatting.RED;
import static net.minecraft.core.net.command.TextFormatting.RESET;
import static net.minecraft.core.util.helper.Direction.*;
import static teamport.aether.AetherMod.TRANSLATOR;
import static teamport.aether.helper.unboxed.IntPair.ipair;
import static teamport.aether.world.generate.feature.components.WorldFeaturePoint.wfp;

public class MobMimic extends MobMonsterAether implements Enemy, AetherDeathMessage {
    public int mimicTime;
    int mimicChestID = AetherBlocks.CHEST_MIMIC_SKYROOT.id();
    int mimicChestMetadata = 0;

    public MobMimic(World world) {
        super(world);
        this.setSize(1.0F, 2.0F);
        this.attackStrength = 5;
        this.scoreValue = 2000;
        this.mimicTime = 60 * Global.TICKS_PER_SECOND; //temp set to 2, was 120
        this.textureIdentifier = NamespaceID.getPermanent("aether", "mimic");
        this.setSkinVariant(this.getSkinVariant());
    }

    public void setBlockData(int mimicChestID, int mimicChestMetadata) {
        this.mimicChestID = mimicChestID;
        this.mimicChestMetadata = mimicChestMetadata;
    }

    public void setChatColor(byte chatColor) {
        this.chatColor = chatColor;
    }

    @Override
    public void addAdditionalSaveData(@NotNull CompoundTag tag) {
        super.addAdditionalSaveData(tag);
        tag.putInt("MimicChestID", mimicChestID);
        tag.putInt("MimicChestMetadata", mimicChestMetadata);
    }

    @Override
    public void readAdditionalSaveData(@NotNull CompoundTag tag) {
        super.readAdditionalSaveData(tag);
        this.mimicChestID = tag.getShort("MimicChestID");
        this.mimicChestMetadata = tag.getShort("MimicChestMetadata");
    }

    @Override
    public void dropDeathItems() {
        MimicEntry variant = MimicRegistry.getMimicVariantByID(this.getSkinVariant());
        this.dropItem(new ItemStack(variant.getChestID(), 1, variant.getChestMetadata()), 0);
        for (WeightedRandomLootObject lootObject : mobDrops) {
            ItemStack stack = lootObject.getDefinedItemStack();
            if (stack != null) {
                this.dropItem(stack, 0);
            }
        }
    }

    @Override
    public void updateAI() {
        super.updateAI();
        if (target == null) {
            if (mimicTime-- == 0) {
                this.remove();
            }
        }
    }

    @Override
    public Entity findPlayerToAttack() {
        assert this.world != null;
        Player player = this.world.getClosestPlayerToEntity(this, 64.0);
        if (player == null || !this.canEntityBeSeen(player) || !player.getGamemode().areMobsHostile()) {
            return null;
        }
        if (!(player instanceof AetherInvisibility)) {
            return player;
        }
        AetherInvisibility invPlayer = (AetherInvisibility) player;
        if (invPlayer.aether$isInvisible()) {
            Player newPlayer = this.world.getClosestPlayerToEntity(this, 2.0);
            if (newPlayer == null || !this.canEntityBeSeen(newPlayer) || !newPlayer.getGamemode().areMobsHostile()) {
                return null;
            }
            return newPlayer;
        }
        return player;
    }

    public void attackEntity(@NotNull Entity entity, float distance) {
        if (this.attackTime <= 0 && distance < 2.0F && entity.bb.maxY > this.bb.minY && entity.bb.minY < this.bb.maxY) {
            this.attackTime = 20;

            int attack = this.attackStrength;
            if (isWallace()) attack = (int) (attack * 1.5);

            entity.hurt(this, attack, DamageType.COMBAT);
        }

    }

    @Override
    public boolean hurt(Entity attacker, int damage, DamageType type) {
        if (damage > 0 && this.isWallace()) damage = Math.max(1, damage / 3);
        return super.hurt(attacker, this.extraDamage(attacker, damage), type);
    }

    public int extraDamage(Entity attacker, int damage) {
        if (!(attacker instanceof Player)) {
            return damage;
        }
        ItemStack item = ((Player) attacker).inventory.getCurrentItem();
        if (item == null) {
            return damage;
        }
        Block<?> block = Blocks.getBlock(this.mimicChestID);
        if (block == null) {
            return damage;
        }
        if (block.hasTag(AetherBlockTags.MINEABLE_BY_AETHER_AXE)
                && (item.getItem() instanceof ItemToolAxe || item.getItem() instanceof ItemToolAxeAether)
        ) {
            return damage << 1;
        }
        if (block.hasTag(AetherBlockTags.MINEABLE_BY_AETHER_PICKAXE)
                && (item.getItem() instanceof ItemToolPickaxe || item.getItem() instanceof ItemToolPickaxeAether)) {
            return damage << 1;
        }
        return damage;
    }

    @Override
    public String getHurtSound() {
        Block<?> block = Blocks.getBlock(mimicChestID);
        if (block == null) {
            return "step.wood";
        }
        return block.getSound().getStepSoundName();
    }

    @Override
    public String getDeathSound() {
        Block<?> block = Blocks.getBlock(mimicChestID);
        Material material = block == null ? Material.wood : block.getMaterial();
        if (material == Material.stone) {
            return "step.stone";
        }
        return "random.door_open";
    }

    @Override
    public float getSoundVolume() {
        return 0.6F;
    }

    @Override
    public int getMaxHealth() {
        return 80;
    }

    public void setLoot(List<ItemStack> loot) {
        if (loot == null || loot.isEmpty()) return;
        for (ItemStack itemStack : loot) {
            mobDrops.add(new WeightedRandomLootObject(itemStack));
        }
    }

    @Override
    public void remove() {
        if (this.isAlive()) {
            place();
        }
        super.remove();
    }

    @Override
    public String deathMessage(Player player) {
        String key = EntityDispatcher.nameKeyForClass(((Entity) this).getClass()) + ".death_message";
        String deathMessage = TRANSLATOR
                .translateKey(key)
                .replace("[PLAYER]", RESET + String.format("<%s>", player.getDisplayName()) + RESET + RED);
        return RED + deathMessage;
    }

    @Override
    public float getHeadHeight() {
        return this.bbHeight;
    }

    private void place() {
        WorldFeaturePoint point = wfp((int) Math.round(this.x), (int) Math.round(this.y), (int) Math.round(this.z));
        Direction[] check = new Direction[]{NONE, NORTH, EAST, SOUTH, WEST, UP, DOWN};
        for (Direction dir : check) {
            WorldFeaturePoint dPoint = point.copy().moveInDirection(dir);
            if (this.isSafe(world, dPoint)) {
                this.placeChest(dPoint);
                this.populateChest(dPoint);
                return;
            }
        }
        this.placeChest(point);
        this.populateChest(point);
    }

    private boolean isSafe(@Nullable World world, WorldFeaturePoint point) {
        Block<?> block = world.getBlock(point.x, point.y, point.z);
        int blockID = block == null ? 0 : block.id();
        Material blockMaterial = blockID == 0 ? Material.air : block.getMaterial();
        return blockID == 0 || blockMaterial.isLiquid();
    }

    private void placeChest(WorldFeaturePoint point) {
        IntPair blockAndMeta = getTarget(world, point);
        world.setBlockAndMetadataWithNotify(point.x, point.y, point.z, blockAndMeta.first, blockAndMeta.second);
        BlockLogicChestMimic.setDefaultDirection(world, point.x, point.y, point.z);
        TileEntity tileEntity = world.getTileEntity(point.x, point.y, point.z);
        if (tileEntity instanceof TileEntityMimic)
            ((TileEntityMimic) tileEntity).setCustomName(this.nickname, this.chatColor);
    }

    private IntPair getTarget(World world, WorldFeaturePoint point) {
        Map<WorldFeaturePoint, Integer> distance = new HashMap<>();
        Queue<WorldFeaturePoint> queue = new ArrayDeque<>();
        Direction[] check = new Direction[]{NORTH, EAST, SOUTH, WEST, UP, DOWN};
        queue.add(point);
        distance.put(point, 0);
        while (!queue.isEmpty()) {
            WorldFeaturePoint next = queue.poll();
            int cdist = distance.get(next);
            if (cdist >= 5) break;
            for (Direction direction : check) {
                WorldFeaturePoint to = new WorldFeaturePoint(next.x + direction.getOffsetX(), next.y + direction.getOffsetY(), next.z + direction.getOffsetZ());
                if (distance.getOrDefault(to, -1) != -1) continue;
                distance.put(to, cdist + 1);
                Block<?> block = world.getBlock(to.x, to.y, to.z);
                int metadata = world.getBlockMetadata(to.x, to.y, to.z);
                BlockLogic blockLogic = block == null ? null : block.getLogic();
                if (blockLogic instanceof BlockLogicChestMimic) {
                    return ipair(block.id(), metadata);
                }
                if (blockLogic instanceof BlockLogicChest) {
                    MimicEntry variant = MimicRegistry.getMimicVariantByChest(block.id(), metadata & 240);
                    return ipair(variant.getMimicChestId(), variant.getMimicChestMetadata());
                }
                queue.add(to);
            }
        }
        MimicEntry variant = MimicRegistry.getMimicVariantByID(this.getSkinVariant());
        return ipair(variant.getMimicChestId(), variant.getMimicChestMetadata());
    }

    private void populateChest(WorldFeaturePoint point) {
        Container inventory = BlockLogicChest.getInventory(world, point.x, point.y, point.z);
        if (inventory == null) {
            return;
        }
        List<WeightedRandomLootObject> listLootObj = this.getMobDrops();
        for (WeightedRandomLootObject lootObj : listLootObj) {
            WorldFeatureComponent.placeItemInChest(random, lootObj.getDefinedItemStack(), inventory);
        }
    }

    public boolean isWallace() {
        MimicEntry variantWallace = MimicRegistry.getMimicVariantByChest(
                AetherBlocks.CHEST_PLANKS_SKYROOT_PAINTED.id(),
                AetherBlocks.CHEST_PLANKS_SKYROOT_PAINTED.getLogic().toMetadata(DyeColor.PURPLE)
        );

        return "Wallace".equals(nickname)
                && variantWallace.getMimicVariant() == this.getSkinVariant();
    }
}
